#! /bin/sh
#
# Copyright by The HDF Group.
# All rights reserved.
#
# More tests for the HDF NPP aggregation tool
# These are additional tests that require more reosurces such as:
# larger or more input data files;
# more disk space;
# longer execution time.
# The tests will run if test data directory is avaiable and $NAGG_TEST_MORE
# is defined.


# The build (current) directory might be different than the source directory.
if test -z "$srcdir"; then
    srcdir=.
fi

# Include Nagg test functions.
. $srcdir/nagg_functions.sh

# Main body.
dir=$SRC_TOOL_TESTFILES/extensive_data

# Continue only if $NAGG_TESST_MORE is defined and test data directory is
# available.
if [ -z "$NAGG_TESST_MORE" ]; then
    echo "\$NAGG_TESST_MORE is not defined."
    echo "Tests skipped."
    exit $EXIT_SUCCESS
fi
if [ ! -d $dir ]; then
    echo "Test data directory ($dir) is not available."
    echo "Tests skipped."
    exit $EXIT_SUCCESS
fi

# ===================================================
# The following tests involve many input files and takes a long time to
# run.  Verify the number of input files before launching the long running
# tests.
# ===================================================
#
# Test many input files for one product, SVM01.
# Verify number of input files
nfiles=120
dfiles=$dir/122293334/SVM01*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT manyinputfilesSVM01.txt -n 16 -t SVM01 $dfiles
fi

# Test several input files for compatible products SVM01-SVM16 and IICMO 
# with their geolocation product GMODO.  The granule duration for IICMO
# was originally incorrect which caused the expected ouput of four granules 
# per each product in one file to be spread over 3 files instead.
# Also, Paul Martinolich, who provided the files, discovered that if -g no 
# is used and GMODO is added to the product list, the geolocation files can
# be added to the data product files at a later time.  Since we only have
# one designated GEO product for each data product, but there are apparently
# multiple valid options (GMTCO) this behavior is desirable.  If this test 
# fails because the unintended behavior is somehow disallowed, another 
# option should be provided.
#
# Verify number of input files
nfiles=72
dfiles=$dir/PaulM-2012-07-30/*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT inputfilesSVM01-16-IICMO-GMODO.txt -n 4 -g no -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16,IICMO,GMODO $dfiles
fi
 

# Test many many input files for products, SVM01-SVM16
nfiles=1885
dfiles=$dir/122293334/SVM*.h5
echo ============================================
echo This takes $nfiles input files
echo ============================================
nx=`echo $dfiles | wc -w`
if [ $nx -ne $nfiles ]; then
    echo "number of input files ($nx) is not as expected ($nfiles)."
    nerrors=`expr $nerrors + 1`
else
    CHECK_OUTPUT manyinputfilesSVM01-16.txt -n 16 -t SVM01,SVM02,SVM03,SVM04,SVM05,SVM06,SVM07,SVM08,SVM09,SVM10,SVM11,SVM12,SVM13,SVM14,SVM15,SVM16 $dfiles
fi
 

# Post results and exit.
echo
if test $nerrors -eq 0 ; then
    echo "All $TESTNAME tests passed."
    exit $EXIT_SUCCESS
else
    echo "$TESTNAME tests failed with $nerrors errors."
    exit $EXIT_FAILURE
fi
